@@ -1,21 +1,7 @@
 /**
- * Copyright 2013 Albert Vaca <albertvaka@gmail.com>
+ * SPDX-FileCopyrightText: 2013 Albert Vaca <albertvaka@gmail.com>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of
- * the License or (at your option) version 3 or any later version
- * accepted by the membership of KDE e.V. (or its successor approved
- * by the membership of KDE e.V.), which shall act as a proxy
- * defined in Section 14 of version 3 of the license.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <https://www.gnu.org/licenses/>.
+ * SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
 #include "pausemusic_config.h"
@@ -25,8 +11,8 @@
 
 K_PLUGIN_FACTORY(PauseMusicConfigFactory, registerPlugin<PauseMusicConfig>();)
 
-PauseMusicConfig::PauseMusicConfig(QWidget* parent, const QVariantList& args)
-    : KdeConnectPluginKcm(parent, args, QStringLiteral("kdeconnect_pausemusic_config"))
+PauseMusicConfig::PauseMusicConfig(QWidget *parent, const QVariantList &args)
+    : KdeConnectPluginKcm(parent, args, QStringLiteral("kdeconnect_pausemusic"))
     , m_ui(new Ui::PauseMusicConfigUi())
 {
     m_ui->setupUi(this);
@@ -57,16 +43,16 @@ void PauseMusicConfig::defaults()
 void PauseMusicConfig::load()
 {
     KCModule::load();
-    bool talking = config()->get(QStringLiteral("conditionTalking"), false);
+    bool talking = config()->getBool(QStringLiteral("conditionTalking"), false);
     m_ui->rad_talking->setChecked(talking);
     m_ui->rad_ringing->setChecked(!talking);
 
-    bool pause = config()->get(QStringLiteral("actionPause"), true);
-    bool mute = config()->get(QStringLiteral("actionMute"), false);
+    bool pause = config()->getBool(QStringLiteral("actionPause"), true);
+    bool mute = config()->getBool(QStringLiteral("actionMute"), false);
     m_ui->check_pause->setChecked(pause);
     m_ui->check_mute->setChecked(mute);
 
-    const bool autoResume = config()->get(QStringLiteral("actionResume"), true);
+    const bool autoResume = config()->getBool(QStringLiteral("actionResume"), true);
     m_ui->check_resume->setChecked(autoResume);
 
     Q_EMIT changed(false);
