@@ -1,26 +1,12 @@
 /**
- * Copyright 2018 Simon Redman <simon@ergotech.com>
+ * SPDX-FileCopyrightText: 2018 Simon Redman <simon@ergotech.com>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of
- * the License or (at your option) version 3 or any later version
- * accepted by the membership of KDE e.V. (or its successor approved
- * by the membership of KDE e.V.), which shall act as a proxy
- * defined in Section 14 of version 3 of the license.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <https://www.gnu.org/licenses/>.
+ * SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
 #include "conversationsdbusinterface.h"
-#include "interfaces/dbusinterfaces.h"
 #include "interfaces/conversationmessage.h"
+#include "interfaces/dbusinterfaces.h"
 
 #include "requestconversationworker.h"
 
@@ -31,12 +17,11 @@
 
 #include "kdeconnect_conversations_debug.h"
 
-QMap<QString, ConversationsDbusInterface*> ConversationsDbusInterface::liveConversationInterfaces;
+QMap<QString, ConversationsDbusInterface *> ConversationsDbusInterface::liveConversationInterfaces;
 
-ConversationsDbusInterface::ConversationsDbusInterface(KdeConnectPlugin* plugin)
-    : QDBusAbstractAdaptor(const_cast<Device*>(plugin->device()))
+ConversationsDbusInterface::ConversationsDbusInterface(KdeConnectPlugin *plugin)
+    : QDBusAbstractAdaptor(const_cast<Device *>(plugin->device()))
     , m_device(plugin->device()->id())
-    , m_plugin(plugin)
     , m_lastId(0)
     , m_smsInterface(m_device)
 {
@@ -44,9 +29,9 @@ ConversationsDbusInterface::ConversationsDbusInterface(KdeConnectPlugin* plugin)
 
     // Check for an existing interface for the same device
     // If there is already an interface for this device, we can safely delete is since we have just replaced it
-    const auto& oldInterfaceItr = ConversationsDbusInterface::liveConversationInterfaces.find(m_device);
+    const auto &oldInterfaceItr = ConversationsDbusInterface::liveConversationInterfaces.find(m_device);
     if (oldInterfaceItr != ConversationsDbusInterface::liveConversationInterfaces.end()) {
-        ConversationsDbusInterface* oldInterface = oldInterfaceItr.value();
+        ConversationsDbusInterface *oldInterface = oldInterfaceItr.value();
         oldInterface->deleteLater();
         ConversationsDbusInterface::liveConversationInterfaces.erase(oldInterfaceItr);
     }
@@ -74,46 +59,50 @@ QVariantList ConversationsDbusInterface::activeConversations()
     toReturn.reserve(m_conversations.size());
 
     for (auto it = m_conversations.cbegin(); it != m_conversations.cend(); ++it) {
-        const auto& conversation = it.value().values();
+        const auto &conversation = it.value().values();
         if (conversation.isEmpty()) {
             // This should really never happen because we create a conversation at the same time
             // as adding a message, but better safe than sorry
-            qCWarning(KDECONNECT_CONVERSATIONS)
-                    << "Conversation with ID" << it.key() << "is unexpectedly empty";
+            qCWarning(KDECONNECT_CONVERSATIONS) << "Conversation with ID" << it.key() << "is unexpectedly empty";
             break;
         }
-        const QVariant& message = QVariant::fromValue<ConversationMessage>(*conversation.crbegin());
+        const QVariant &message = QVariant::fromValue<ConversationMessage>(*conversation.crbegin());
         toReturn.append(message);
     }
 
     return toReturn;
 }
 
-void ConversationsDbusInterface::requestConversation(const qint64& conversationID, int start, int end)
+void ConversationsDbusInterface::requestConversation(const qint64 &conversationID, int start, int end)
 {
     if (start < 0 || end < 0) {
-        qCWarning(KDECONNECT_CONVERSATIONS) << "requestConversation" << "Start and end must be >= 0";
+        qCWarning(KDECONNECT_CONVERSATIONS) << "requestConversation"
+                                            << "Start and end must be >= 0";
         return;
     }
 
     if (end - start < 0) {
-        qCWarning(KDECONNECT_CONVERSATIONS) << "requestConversation" <<"Start must be before end";
+        qCWarning(KDECONNECT_CONVERSATIONS) << "requestConversation"
+                                            << "Start must be before end";
         return;
     }
 
-    RequestConversationWorker* worker = new RequestConversationWorker(conversationID, start, end, this);
-    connect(worker, &RequestConversationWorker::conversationMessageRead,
-            this, &ConversationsDbusInterface::conversationUpdated,
-            Qt::QueuedConnection);
+    RequestConversationWorker *worker = new RequestConversationWorker(conversationID, start, end, this);
+    connect(worker, &RequestConversationWorker::conversationMessageRead, this, &ConversationsDbusInterface::conversationUpdated, Qt::QueuedConnection);
     worker->work();
 }
 
+void ConversationsDbusInterface::requestAttachmentFile(const qint64 &partID, const QString &uniqueIdentifier)
+{
+    m_smsInterface.getAttachment(partID, uniqueIdentifier);
+}
+
 void ConversationsDbusInterface::addMessages(const QList<ConversationMessage> &messages)
 {
     QSet<qint64> updatedConversationIDs;
 
-    for (const auto& message : messages) {
-        const qint32& threadId = message.threadID();
+    for (const auto &message : messages) {
+        const qint32 &threadId = message.threadID();
 
         // We might discover that there are no new messages in this conversation, thus calling it
         // "updated" might turn out to be a bit misleading
@@ -129,7 +118,7 @@ void ConversationsDbusInterface::addMessages(const QList<ConversationMessage> &m
 
         // Store the Message in the list corresponding to its thread
         bool newConversation = !m_conversations.contains(threadId);
-        const auto& threadPosition = m_conversations[threadId].insert(message.date(), message);
+        const auto &threadPosition = m_conversations[threadId].insert(message.date(), message);
         m_known_messages[threadId].insert(message.uID());
 
         // If this message was inserted at the end of the list, it is the latest message in the conversation
@@ -157,18 +146,18 @@ void ConversationsDbusInterface::addMessages(const QList<ConversationMessage> &m
     waitingForMessagesLock.unlock();
 }
 
-void ConversationsDbusInterface::removeMessage(const QString& internalId)
+void ConversationsDbusInterface::removeMessage(const QString &internalId)
 {
     // TODO: Delete the specified message from our internal structures
     Q_UNUSED(internalId);
 }
 
-QList<ConversationMessage> ConversationsDbusInterface::getConversation(const qint64& conversationID) const
+QList<ConversationMessage> ConversationsDbusInterface::getConversation(const qint64 &conversationID) const
 {
     return m_conversations.value(conversationID).values();
 }
 
-void ConversationsDbusInterface::updateConversation(const qint64& conversationID)
+void ConversationsDbusInterface::updateConversation(const qint64 &conversationID)
 {
     waitingForMessagesLock.lock();
     if (conversationsWaitingForMessages.contains(conversationID)) {
@@ -179,14 +168,29 @@ void ConversationsDbusInterface::updateConversation(const qint64& conversationID
     }
     qCDebug(KDECONNECT_CONVERSATIONS) << "Requesting conversation with ID" << conversationID << "from remote";
     conversationsWaitingForMessages.insert(conversationID);
-    m_smsInterface.requestConversation(conversationID);
+
+    // Request a window of messages
+    qint64 rangeStartTimestamp;
+    qint64 numberToRequest;
+    if (m_conversations.contains(conversationID) && m_conversations[conversationID].count() > 0) {
+        rangeStartTimestamp = m_conversations[conversationID].first().date(); // Request starting from the oldest-available message
+        numberToRequest = m_conversations[conversationID].count(); // Request an increasing number of messages by requesting more equal to the amount we have
+    } else {
+        rangeStartTimestamp = -1; // Value < 0 indicates to return the newest messages
+        numberToRequest = MIN_NUMBER_TO_REQUEST; // Start off with a small batch
+    }
+    if (numberToRequest < MIN_NUMBER_TO_REQUEST) {
+        numberToRequest = MIN_NUMBER_TO_REQUEST;
+    }
+    m_smsInterface.requestConversation(conversationID, rangeStartTimestamp, numberToRequest);
+
     while (conversationsWaitingForMessages.contains(conversationID)) {
         waitingForMessages.wait(&waitingForMessagesLock);
     }
     waitingForMessagesLock.unlock();
 }
 
-void ConversationsDbusInterface::replyToConversation(const qint64& conversationID, const QString& message)
+void ConversationsDbusInterface::replyToConversation(const qint64 &conversationID, const QString &message, const QVariantList &attachmentUrls)
 {
     const auto messagesList = m_conversations[conversationID];
     if (messagesList.isEmpty()) {
@@ -194,15 +198,19 @@ void ConversationsDbusInterface::replyToConversation(const qint64& conversationI
         return;
     }
 
-    const QList<ConversationAddress>& addressList = messagesList.first().addresses();
-    QVariant addresses;
-    addresses.setValue(addressList);
+    const QList<ConversationAddress> &addressList = messagesList.first().addresses();
+    QVariantList addresses;
 
-    m_smsInterface.sendSms(QDBusVariant(addresses), message, messagesList.first().subID());
+    for (const auto &address : addressList) {
+        addresses << QVariant::fromValue(address);
+    }
+
+    m_smsInterface.sendSms(addresses, message, attachmentUrls, messagesList.first().subID());
 }
 
-void ConversationsDbusInterface::sendWithoutConversation(const QDBusVariant& addresses, const QString& message) {
-    m_smsInterface.sendSms(addresses, message);
+void ConversationsDbusInterface::sendWithoutConversation(const QVariantList &addresses, const QString &message, const QVariantList &attachmentUrls)
+{
+    m_smsInterface.sendSms(addresses, message, attachmentUrls);
 }
 
 void ConversationsDbusInterface::requestAllConversationThreads()
@@ -215,3 +223,8 @@ QString ConversationsDbusInterface::newId()
 {
     return QString::number(++m_lastId);
 }
+
+void ConversationsDbusInterface::attachmentDownloaded(const QString &filePath, const QString &fileName)
+{
+    Q_EMIT attachmentReceived(filePath, fileName);
+}
